class AutoInjector {
  final _map = <String, Function>{};

  void add<T>(Function constructor) {
    var type = T.toString();

    if (['dynamic', 'Object', 'Object?'].contains(type)) {
      type = RegExp(r"\'(.+)\.\'").firstMatch(constructor.toString())!.group(1)!;
    }

    if (hasInjection<T>()) {
      throw Exception('');
    }

    _map[type] = constructor;
  }

  void remove<T>() {
    var type = T.toString();
    _map.remove(type);
  }

  bool hasInjection<T>() => _map.containsKey(T.toString());

  dynamic _getByClassName(String className) {
    final constructor = _map[className];

    if (constructor.toString().startsWith('Closure: () =>')) {
      return constructor?.call();
    }

    final regex = RegExp(r'Closure: \((.+)\) => .+');
    final args = _InjectArgs();

    final match = regex.firstMatch(constructor.toString());

    var inputText = match!.group(1)!;

    final hasNamedParams = RegExp(r'\{(.+)\}');
    final namedParams = hasNamedParams.firstMatch(inputText);

    if (namedParams != null) {
      final named = namedParams.group(1)!;
      inputText = inputText.replaceAll('{$named}', '');

      final params = named.split(',').map((e) => e.trim()).toList();

      for (var param in params) {
        final anatomicParam = param.split(' ');

        if (anatomicParam.first == 'required') {
          args._namedArguments[Symbol(anatomicParam.last)] = _getByClassName(anatomicParam[1].replaceFirst('?', ''));
        } else {
          args._namedArguments[Symbol(anatomicParam.last)] = _getByClassName(anatomicParam[0].replaceFirst('?', ''));
        }
      }
    }

    if (inputText.isNotEmpty) {
      final objects = inputText //
          .split(',')
          .map((e) => e.trim())
          .map(_getByClassName)
          .toList();

      args._positionalArguments.addAll(objects);
    }

    return Function.apply(constructor!, args._positionalArguments, args._namedArguments);
  }

  T? get<T>() {
    return _getByClassName(T.toString());
  }
}

class _InjectArgs {
  final List<dynamic> _positionalArguments = [];
  final Map<Symbol, dynamic> _namedArguments = {};
}
